package com.jd.dynamic.apis

import android.app.Activity
import android.app.Dialog
import android.content.Context
import android.os.AsyncTask
import android.os.Build
import android.text.TextUtils
import android.util.AttributeSet
import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.FrameLayout
import androidx.annotation.Keep
import androidx.annotation.RequiresApi
import com.jd.dynamic.DYConstants
import com.jd.dynamic.R
import com.jd.dynamic.base.DynamicMtaUtil
import com.jd.dynamic.base.DynamicSdk
import com.jd.dynamic.base.DynamicTemplateEngine
import com.jd.dynamic.base.NewDynamicFetcher
import com.jd.dynamic.base.interfaces.IExceptionHandler
import com.jd.dynamic.entity.ResultEntity
import com.jd.dynamic.entity.ViewNode
import com.jd.dynamic.lib.ab.DYABConfigUtil
import com.jd.dynamic.lib.common.Constants
import com.jd.dynamic.lib.dynamic.cache.NewCacheManager
import com.jd.dynamic.lib.dynamic.parser.NewDynamicXParser
import com.jd.dynamic.lib.error.DynamicException
import com.jd.dynamic.lib.localRes.LocalTemplateManager
import com.jd.dynamic.lib.utils.CommonUtil
import com.jd.dynamic.lib.utils.DPIUtil
import com.jd.dynamic.lib.utils.DigestUtils
import com.jd.dynamic.lib.utils.LogUtil
import com.jd.dynamic.lib.viewparse.BaseFrameLayout
import com.jd.dynamic.lib.views.skeleton.shimmer.Shimmer
import com.jd.dynamic.lib.views.skeleton.shimmer.ShimmerFrameLayout
import com.jd.dynamic.yoga.android.YogaLayout
import org.json.JSONObject
import java.io.File

/**
 * @author: xuepanjie1
 * @date:2022/3/17
 * @description:
 */
@Keep
class DynamicContainer : FrameLayout {

    constructor(context: Context) : this(context, null)

    constructor(context: Context, attrs: AttributeSet?) : this(context, attrs, 0)

    constructor(context: Context, attrs: AttributeSet?, defStyleAttr: Int) : super(
        context,
        attrs,
        defStyleAttr
    )
    fun onViewRecycled(){
        engine?.releaseTimers(false)
        if(CommonUtil.nonEmpty(engine?.cachePool?.recycleListeners)){
            engine?.cachePool?.recycleListeners?.forEach { it.onViewRecycled() }
        }
    }
    @RequiresApi(Build.VERSION_CODES.LOLLIPOP)
    constructor(
        context: Context,
        attrs: AttributeSet?,
        defStyleAttr: Int,
        defStyleRes: Int
    ) : super(context, attrs, defStyleAttr, defStyleRes)


    private var mDialog: Dialog? = null

    private var containerData: JSONObject? = null
    private var engine: DynamicTemplateEngine? = null
    private var containerListener: IDYContainerListener? = null
    private var containerConfig: DYContainerConfig? = null
    private var onAddDynamicViewListener:OnAddDynamicViewListener?=null
    private var skeletonLayout: View? = null
    private var shimmerFrameLayout: ShimmerFrameLayout? = null
    private var isAsyncCreateView:Boolean = false
    fun attachConfig(config: DYContainerConfig) {
        containerConfig = config
        if(containerConfig?.isUseSkeleton() == true){
            skeletonLayout = LayoutInflater.from(context).inflate(R.layout.dyn_item_skeleton_layout, null)
            skeletonLayout?.visibility = GONE
            shimmerFrameLayout = skeletonLayout!!.findViewById(R.id.shimmer_view_container)
            shimmerFrameLayout?.shimmer = Shimmer.AlphaHighlightBuilder().setWidthRatio(1.5f).build()
            val params = LayoutParams(LayoutParams.MATCH_PARENT, DPIUtil.dip2px(containerConfig!!.getSkeletonHeight().toFloat()))
            params.gravity = Gravity.CENTER
            addView(skeletonLayout, params)
        }else{
            checkAndPost(Runnable { prepareContainer() })
        }
    }

    fun attachListener(listener: IDYContainerListener) {
        containerListener = listener
    }

    fun setData(data: JSONObject? = null) {
        containerData = data
    }

    fun getData(): JSONObject? = containerData

    private fun isConfigInvalid(): Boolean {
        return containerConfig == null
    }

    fun setDialog(dialog: Dialog? = null) {
        this.mDialog = dialog
    }

    /**
     * 0. 内存
     * 1. 磁盘缓存
     * 2. 兜底 & 请求网络
     * */
    // 仅仅从内存判断是否有 template 没有则加载失败。
    fun load(): Boolean {
        if (isConfigInvalid()) {
            return false
        }
        val loadTime = System.nanoTime()
        DynamicMtaUtil.reportAPIEnter(containerConfig?.module, containerConfig?.templateId, "load")

        val mtaId = DynamicMtaUtil.startMtaStat(
            DynamicSdk.getEngine().appType,
            containerConfig?.module,
            DYConstants.DY_EMPTY_STR,
            containerConfig?.templateId,
            false
        )

        if (engine != null && TextUtils.equals(containerConfig?.templateId, engine!!.bizField)) {
            checkAndPost(Runnable { this.visibility = VISIBLE })
            engine!!.newRefreshView(containerData)
            containerListener?.onRefresh()
            return true
        } else if(DYABConfigUtil.getInstance().isForceUseBackUpInDebug
                && NewCacheManager.hasLocalBackTempFile(containerConfig?.module,
                        containerConfig?.templateId)
        ){ //强制兜底
            DynamicMtaUtil.startGetTemplate(
                    containerConfig?.module,
                    containerConfig?.templateId,
                    mtaId
            )
            //加载兜底
            val entity = LocalTemplateManager.loadLocalTempFile(
                    containerConfig?.module,
                    containerConfig?.templateId,
                    Constants.HTTPResCode.CODE_BACKUP_FROM_NEW_API,
                    mtaId
            )
            if (CommonUtil.isTemplateValid(entity)) {
                DynamicSdk.handException(
                    IExceptionHandler.DynamicExceptionData.TYPE_LOCAL_BACKUP,
                    "DynamicContainer.load() 强制使用兜底",
                    containerConfig?.templateId,
                    containerConfig?.module,
                    Constants.LOAD_BACKUP_TYPE_FORCE_USE_DEBUG,
                    null
                )
                showViewNode(
                        entity!!.viewNode,
                        mtaId,
                        entity,
                        source = "backUp",
                        time = loadTime
                )
                return true
            }

        }else if (NewCacheManager.hasTempFile(
                containerConfig?.module,
                containerConfig?.templateId
            )
        ) {
            // 磁盘
            // 有问题，这里仅仅提供 template ，如果是 zip 应该是系列的东西， zip 的磁盘缓存应该是 zip ，或者这里抽象通用的东西
            val template = NewCacheManager.getTemplateByBizField(
                containerConfig?.module,
                containerConfig?.templateId
            )
            template?.let {


                DynamicMtaUtil.updateTemplate(mtaId,it)

                val digestFileName = DigestUtils.getDigest(
                    it.downloadUrl,
                    it.downloadFileName
                )
                val cacheFile = File(
                    NewCacheManager.getSysCodeAbsPath(containerConfig?.module) +
                            File.separator + it.businessCode,
                    digestFileName
                )

                if (cacheFile.exists()) {
                    DynamicMtaUtil.startGetTemplate(
                        containerConfig?.module,
                        containerConfig?.templateId,
                        mtaId
                    )
                    //TODO 有问题，应该加载出ResultEntity对象，不然zip会有问题
                    val entity = NewDynamicXParser.parseBinaryToResultEntity(
                        cacheFile.absolutePath,
                        false,
                        containerConfig?.templateId,
                        containerConfig?.module,
                        mtaId
                    )
                    if (CommonUtil.isTemplateValid(entity)) {
                        showViewNode(
                            entity.viewNode,
                            mtaId,
                            entity,
                            source = "diskCache",
                            time = loadTime
                        )
                        return true
                    }
                } else {
                    return false
                }
            }
        } else {
            // 请求网络
            containerConfig?.module?.let {
                DynamicSdk.getDriver().prepare(it, DYConstants.DY_EMPTY_STR)
            }
            // 兜底
            if (NewCacheManager.hasLocalBackTempFile(
                    containerConfig?.module,
                    containerConfig?.templateId
                )
            ) {
                DynamicMtaUtil.startGetTemplate(
                    containerConfig?.module,
                    containerConfig?.templateId,
                    mtaId
                )
                //加载兜底
                val entity = LocalTemplateManager.loadLocalTempFile(
                    containerConfig?.module,
                    containerConfig?.templateId,
                    Constants.HTTPResCode.CODE_BACKUP_FROM_NEW_API,
                    mtaId
                )
                if (CommonUtil.isTemplateValid(entity)) {
                    DynamicSdk.handException(
                        IExceptionHandler.DynamicExceptionData.TYPE_LOCAL_BACKUP,
                        "DynamicContainer.load() 加载兜底",
                        containerConfig?.templateId,
                        containerConfig?.module,
                        Constants.LOAD_BACKUP_TYPE_IN_LOAD,
                        null
                    )
                    showViewNode(
                        entity!!.viewNode,
                        mtaId,
                        entity,
                        source = "backUp",
                        time = loadTime
                    )
                    return true
                }
            }
            return false
        }
        return false
    }

    fun findChildViewById(vid: Int): View? {
        return try {
            engine?.rootContainer?.findViewById(vid)
        } catch (e: java.lang.Exception) {
            null
        }
    }

    private fun showViewNode(
        viewNode: ViewNode, mtaId: String?, obj: ResultEntity? = null,
        source: String, time: Long
    ) {
        val enterShow = System.nanoTime()
        val loadTime = DynamicMtaUtil.getCurMicroseconds(enterShow - time)
        DYConstants.DYLog("showViewNode start time :$enterShow time is: $time load time: $loadTime source: $source")
        DynamicMtaUtil.reportAPILoad(
            containerConfig?.module,
            containerConfig?.templateId,
            source,
            loadTime
        )

        checkAndPost(Runnable {

            this.visibility = VISIBLE

        })

        val pkgName = if (containerConfig?.getPackageName()
                .isNullOrEmpty()
        ) DYConstants.DY_EMPTY_STR else containerConfig?.getPackageName()
        engine = DynamicTemplateEngine(
            pkgName,
            context as Activity,
            this,
            containerConfig?.factory
        )

        obj?.let {
            engine?.entity = obj
        }
        engine!!.containerWidth = containerConfig?.getContainerWidth()!!
        engine!!.containerHeight = containerConfig?.getContainerHeight()!!
        engine!!.dialog = mDialog;
        engine!!.bizField = containerConfig?.templateId
        engine!!.setSystemCode(containerConfig?.module)
        engine!!.useAsyncItem = containerConfig?.isUseAsyncItem()
        engine!!.useTagViewOptimize = containerConfig?.isUseTagViewOptimize() == true
        engine!!.shouldAutoListenDarkStatus(containerConfig?.isAutoListenDarkStatus()!!)
        engine!!.newInitTemplate(
            viewNode,
            containerData,
            mtaId
        )
        containerListener?.onLoad()
        val renderTime = DynamicMtaUtil.getCurMicroseconds(System.nanoTime() - enterShow)
        DynamicMtaUtil.reportAPIRender(
            containerConfig?.module,
            containerConfig?.templateId,
            source,
            renderTime
        )
        DYConstants.DYLog("showViewNode render time : $renderTime")
    }

    private fun prepareContainer() {
        if (childCount > 0) {
            removeAllViews()
        }
    }

    // todo check if need load xml?
    fun load(callback: IContainerCallback) {
        if (isConfigInvalid()) {
            callback.onError(DynamicException("config is invalid."))
            return
        }
        DynamicMtaUtil.reportAPIEnter(
            containerConfig?.module,
            containerConfig?.templateId,
            "load async"
        )
        if (engine != null && TextUtils.equals(containerConfig?.templateId, engine!!.bizField)) {
            checkAndPost(Runnable { this.visibility = VISIBLE })

            engine!!.newRefreshView(containerData)
            containerListener?.onRefresh()
            callback.onSuccess()
        } else {
            try {
                val beginTime = System.currentTimeMillis()

                NewDynamicFetcher.requestDynamicConfigByBizField(true,
                    containerConfig?.module,
                    containerConfig?.templateId,
                    DYConstants.DY_EMPTY_STR,
                    object : NewDynamicFetcher.Listener2() {
                        override fun onStart() {

                        }

                        override fun onError(e: java.lang.Exception?) {
                            if (e is DynamicException) {
                                callback.onError(e)
                            } else {
                                callback.onError(DynamicException(e?.message))
                            }
                        }

                        override fun onEnd(obj: ResultEntity?, mtaId: String?) {
                            if (obj != null) {
//                                if (true == containerConfig?.getUseCustomBackup()) {
//                                    if (obj.isBackup) {
//                                        return callback.onError(DynamicException(DYConstants.DY_EMPTY_STR))
//                                    }
//                                }
                                showViewNode(obj.viewNode, mtaId, obj, "network", beginTime)
                                callback.onSuccess()
                            } else {
                                callback.onError(DynamicException(DYConstants.DY_EMPTY_STR))
                            }
                        }
                    }
                )
            } catch (e: Exception) {
                if (e is DynamicException) {
                    callback.onError(e)
                } else {
                    callback.onError(DynamicException(e.message))
                }
            }
        }
    }

    // must update data.
    fun refresh(): Boolean {
        if (isConfigInvalid()) {
            return false
        }
        DynamicMtaUtil.reportAPIEnter(
            containerConfig?.module,
            containerConfig?.templateId,
            "refresh"
        )
        if (engine != null && TextUtils.equals(containerConfig?.templateId, engine!!.bizField)) {
            checkAndPost(Runnable { this.visibility = VISIBLE })
            engine!!.newRefreshView(containerData)
            containerListener?.onRefresh()
            return true
        }
        return false
    }

    private fun checkAndPost(runnable: Runnable) {
        if (CommonUtil.isMainThread()) {
            runnable.run()
        } else {
            (containerConfig?.context as? Activity)?.runOnUiThread(runnable)
        }
    }
    private fun hideOrShowSkeletonView(show:Boolean){
        if(show){
            skeletonLayout?.visibility = VISIBLE
            shimmerFrameLayout?.showShimmer(true)
        }else{
            skeletonLayout?.visibility = GONE
            shimmerFrameLayout?.hideShimmer()
        }
    }
    /**
     * 异步绑定数据
     */
    fun bindViewAsync(listener: IViewBindListener?){
        LogUtil.e("bindViewData","engine is null "+(engine==null),"rootView is null "+(findViewById<View>(R.id.dynamic_root_view)==null))
        hideOrShowSkeletonView(true)
        if (engine != null) {
            if(findViewById<View>(R.id.dynamic_root_view)!=null){
                engine!!.bindViewAsync(containerData,object:IViewBindListener{
                    override fun onDynamicViewBind() {
                        if(isAsyncCreateView){
                            //异步创建的先不可见，等待数据绑定完成
                            findViewById<BaseFrameLayout>(R.id.dynamic_root_view)?.visibility = VISIBLE
                        }
                        hideOrShowSkeletonView(false)
                        listener?.onDynamicViewBind()
                    }

                    override fun onError(exception: DynamicException) {
                        hideOrShowSkeletonView(false)
                        DynamicSdk.handException(IExceptionHandler.DynamicExceptionData.TYPE_BIND_ASYNC,exception.message,containerConfig?.templateId,containerConfig?.module,Constants.HTTPResCode.CODE_BIND_ASYNC_ALL,exception)
                        listener?.onError(exception)
                    }

                })
            }else{
                this.onAddDynamicViewListener = object :OnAddDynamicViewListener{
                    override fun onAddView() {
                        engine!!.bindViewAsync(containerData,object:IViewBindListener{
                            override fun onDynamicViewBind() {
                                if(isAsyncCreateView){
                                    //异步创建的先不可见，等待数据绑定完成
                                    findViewById<BaseFrameLayout>(R.id.dynamic_root_view)?.visibility = VISIBLE
                                }
                                hideOrShowSkeletonView(false)
                                listener?.onDynamicViewBind()
                            }

                            override fun onError(exception: DynamicException) {
                                hideOrShowSkeletonView(false)
                                DynamicSdk.handException(IExceptionHandler.DynamicExceptionData.TYPE_BIND_ASYNC,exception.message,containerConfig?.templateId,containerConfig?.module,Constants.HTTPResCode.CODE_BIND_ASYNC_ALL,exception)
                                listener?.onError(exception)
                            }

                        })
                    }
                }
            }
        }else{
            this.onAddDynamicViewListener = object :OnAddDynamicViewListener{
                override fun onAddView() {
                    bindViewAsync(listener)
                }
            }
        }
    }
    /**
     * 同步绑定数据
     */
    fun bindView(){
        hideOrShowSkeletonView(false)
        engine?.bindView(containerData)
    }

    /**
     * 清理动态化缓存节点
     */
    fun resetData(){
        engine?.currentData?.remove(Constants.DYN_PRV_DATA)
        engine?.currentData?.remove(Constants.DYN_PRV_ITEM_DATA)
        engine?.currentData?.remove(Constants.DYN_PRV_JS_DATA)

        containerData?.remove(Constants.DYN_PRV_DATA)
        containerData?.remove(Constants.DYN_PRV_ITEM_DATA)
        containerData?.remove(Constants.DYN_PRV_JS_DATA)
    }
    /**
     * 通过本地模板创建View
     */
    fun createView():Boolean{
        isAsyncCreateView = false
       val task = AsyncCreateViewTask(this,null,null,0L)
        val view = task.doInBackground(containerConfig!!)
        task.onPostExecute(view)
        return view != null
    }
    /**
     * 通过本地模板异步创建View
     */
    fun createViewAsync(listener:IViewCreateListener?){
        if (isConfigInvalid()) {
            listener?.onError(DynamicException("config is null"))
        }
        hideOrShowSkeletonView(true)
        isAsyncCreateView = true
        AsyncCreateViewTask(this,listener,null,0L).execute(containerConfig)
    }
    private fun innerCreateView(
            viewNode: ViewNode, mtaId: String?,obj: ResultEntity? = null,source: String, time: Long) :View{
        val enterShow = System.nanoTime()
        val loadTime = DynamicMtaUtil.getCurMicroseconds(enterShow - time)
        DynamicMtaUtil.reportAPILoad(
                containerConfig?.module,
                containerConfig?.templateId,
                source,
                loadTime
        )
        val pkgName = if (containerConfig?.getPackageName()
                        .isNullOrEmpty()
        ) DYConstants.DY_EMPTY_STR else containerConfig?.getPackageName()
        engine = DynamicTemplateEngine(
                pkgName,
                context as Activity,
                this,
                containerConfig?.factory
        )
        LogUtil.e("createView","new engine","time = "+DynamicMtaUtil.getCurMicroseconds(System.nanoTime() - enterShow))
        var time = System.nanoTime()
        obj?.let {
            engine?.entity = obj
        }
        engine!!.containerWidth = containerConfig?.getContainerWidth()!!
        engine!!.containerHeight = containerConfig?.getContainerHeight()!!
        engine!!.dialog = mDialog;
        engine!!.bizField = containerConfig?.templateId
        engine!!.setSystemCode(containerConfig?.module)
        engine!!.shouldAutoListenDarkStatus(containerConfig?.isAutoListenDarkStatus()!!)
        engine!!.useAsyncItem = containerConfig?.isUseAsyncItem()
        engine!!.useTagViewOptimize = containerConfig?.isUseTagViewOptimize()!!
        LogUtil.e("createView","setting engine","time = "+DynamicMtaUtil.getCurMicroseconds(System.nanoTime() - time))
        time = System.nanoTime()
        val view = engine!!.returnDynamicView(
                viewNode,
                mtaId
        )
        LogUtil.e("createView","engine createView","time = "+DynamicMtaUtil.getCurMicroseconds(System.nanoTime() - time))

        val renderTime = DynamicMtaUtil.getCurMicroseconds(System.nanoTime() - enterShow)
        DynamicMtaUtil.reportAPIRender(
                containerConfig?.module,
                containerConfig?.templateId,
                source,
                renderTime
        )
        return view
    }

    /**
     * 由外部触发刷新暗黑，避免全量刷新
     */
    fun refreshDarkChange(isDark:Boolean){
        engine?.onDarkChange(isDark)
    }
    /**
     * 通过网络模板，异步创建View
     */
    fun createViewWithNetwork(listener: IViewCreateListener?) {
        if (isConfigInvalid()) {
            listener?.onError(DynamicException("config is invalid."))
            return
        }
            try {
                hideOrShowSkeletonView(true)
                val beginTime = System.currentTimeMillis()
                NewDynamicFetcher.requestDynamicConfigByBizField(true,
                        containerConfig?.module,
                        containerConfig?.templateId,
                        DYConstants.DY_EMPTY_STR,
                        object : NewDynamicFetcher.Listener2() {
                            override fun onStart() {

                            }

                            override fun onError(e: java.lang.Exception?) {
                                if (e is DynamicException) {
                                    listener?.onError(e)
                                } else {
                                    listener?.onError(DynamicException(e?.message))
                                }
                            }

                            override fun onEnd(obj: ResultEntity?, mtaId: String?) {
                                if (obj != null) {
                                    AsyncCreateViewTask(this@DynamicContainer,listener,obj,beginTime).execute(containerConfig)
                                } else {
                                    listener?.onError(DynamicException(DYConstants.DY_EMPTY_STR))
                                }
                            }
                        }
                )
            } catch (e: Exception) {
                if (e is DynamicException) {
                    listener?.onError(e)
                } else {
                    listener?.onError(DynamicException(e.message))
                }
            }
        }

    override fun addView(child: View?) {
        super.addView(child)
        if(onAddDynamicViewListener!=null && child?.id == R.id.dynamic_root_view){
            onAddDynamicViewListener?.onAddView()
        }
    }

    fun updateWidthAndHeight(width: Int, height: Int) {
        if (width >= 0) {
            engine?.containerWidth = width
        }
        if (height >= 0) {
            engine?.containerHeight = height
        }
    }

    override fun addView(child: View?, params: ViewGroup.LayoutParams?) {
        super.addView(child, params)
        if(onAddDynamicViewListener!=null && child?.id == R.id.dynamic_root_view){
            onAddDynamicViewListener?.onAddView()
        }
    }
    private interface OnAddDynamicViewListener{
        fun onAddView()
    }
    private class AsyncCreateViewTask(val container: DynamicContainer, val listener: IViewCreateListener?, val entity: ResultEntity?, val beginTime: Long?) : AsyncTask<DYContainerConfig, Void?, View?>() {
        public override fun doInBackground(vararg voids: DYContainerConfig): View? {
            val containerConfig = voids[0]
            var  time = System.nanoTime();
            val cacheKey: String = containerConfig.module + "_" + containerConfig.templateId
            if (containerConfig.isUseCacheView() && CommonUtil.nonEmpty(DynamicSdk.getEngine().templateEngineCache) && CommonUtil.nonEmpty(DynamicSdk.getEngine().templateEngineCache[cacheKey])) {
                val views = DynamicSdk.getEngine().templateEngineCache[cacheKey]
                if (CommonUtil.nonEmpty(views)) {
                    val engine = views!!.removeAt(0)
                    if (engine.rootContainer != null) {
                        container.engine = engine
                        container.engine!!.activity = containerConfig.context as Activity?
                        container.engine!!.customFactory = containerConfig.factory
                        container.engine!!.dialog =container.mDialog
                        container.engine!!.bizField = containerConfig?.templateId
                        container.engine!!.systemCode = containerConfig?.module
                        container.engine!!.useAsyncItem = containerConfig?.isUseAsyncItem()
                        container.engine!!.useTagViewOptimize = containerConfig?.isUseTagViewOptimize() == true
                        container.engine!!.shouldAutoListenDarkStatus(containerConfig?.isAutoListenDarkStatus()!!)
                        container.engine!!.containerWidth = containerConfig?.getContainerWidth()!!
                        container.engine!!.containerHeight = containerConfig?.getContainerHeight()!!
                        container.engine!!.isAttached = true
                        container.engine!!.parseViewListeners()
                        engine.rootContainer.addLifecycleObserver()
                        LogUtil.e("createView", "get cached View time = ", DynamicMtaUtil.getCurMicroseconds(System.nanoTime() - time))
                        return engine.rootContainer
                    }

                }
            }
            var view:View? = null

            val loadTime = System.nanoTime()

            val mtaId = DynamicMtaUtil.startMtaStat(
                    DynamicSdk.getEngine().appType,
                    containerConfig?.module,
                    DYConstants.DY_EMPTY_STR,
                    containerConfig?.templateId,
                    false
            )
            if(entity!=null){
                view = container.innerCreateView(entity.viewNode, mtaId,entity,"network",beginTime!!)
            }else if(DYABConfigUtil.getInstance().isForceUseBackUpInDebug
                    && NewCacheManager.hasLocalBackTempFile(containerConfig?.module,
                            containerConfig?.templateId)){ //强制兜底
                DynamicMtaUtil.startGetTemplate(
                        containerConfig?.module,
                        containerConfig?.templateId,
                        mtaId)
                //加载兜底
                var time  = System.nanoTime();
                val entity = LocalTemplateManager.loadLocalTempFile(
                        containerConfig?.module,
                        containerConfig?.templateId,
                        Constants.HTTPResCode.CODE_BACKUP_FROM_NEW_API,
                        mtaId)
                LogUtil.e("createView","loadLocalTempFile","time = "+DynamicMtaUtil.getCurMicroseconds(System.nanoTime() - time))
                if (CommonUtil.isTemplateValid(entity)) {
                    DynamicSdk.handException(
                        IExceptionHandler.DynamicExceptionData.TYPE_LOCAL_BACKUP,
                        "DynamicContainer.createView 强制使用兜底",
                        containerConfig?.templateId,
                        containerConfig?.module,
                        Constants.LOAD_BACKUP_TYPE_FORCE_USE_DEBUG,
                        null
                    )
                    view = container.innerCreateView(
                            entity!!.viewNode, mtaId,
                            entity,source = "backUp",
                            time = loadTime
                    )
                }

            }else if (NewCacheManager.hasTempFile(
                            containerConfig?.module,
                            containerConfig?.templateId)) {
                val template = NewCacheManager.getTemplateByBizField(
                        containerConfig?.module,
                        containerConfig?.templateId)
                template?.let {

                    DynamicMtaUtil.updateTemplate(mtaId,it)
                    val digestFileName = DigestUtils.getDigest(
                            it.downloadUrl,
                            it.downloadFileName)
                    val cacheFile = File(
                            NewCacheManager.getSysCodeAbsPath(containerConfig?.module) +
                                    File.separator + it.businessCode,
                            digestFileName)
                    if (cacheFile.exists()) {
                        DynamicMtaUtil.startGetTemplate(
                            containerConfig?.module,
                            containerConfig?.templateId,
                            mtaId)
                        val entity = NewDynamicXParser.parseBinaryToResultEntity(
                                cacheFile.absolutePath,
                                false,
                                containerConfig?.templateId,
                                containerConfig?.module,
                                mtaId)

                        if (CommonUtil.isTemplateValid(entity)) {
                            view = container.innerCreateView(
                                    entity!!.viewNode, mtaId,
                                    entity, source = "diskCache",
                                    time = loadTime)
                        }
                    }
                }
            }else {
                DynamicSdk.getDriver().prepare(containerConfig?.module, DYConstants.DY_EMPTY_STR)
                // 兜底
                if (NewCacheManager.hasLocalBackTempFile(
                                containerConfig?.module,
                                containerConfig?.templateId
                        )
                ) {
                    DynamicMtaUtil.startGetTemplate(
                            containerConfig?.module,
                            containerConfig?.templateId,
                            mtaId
                    )
                    //加载兜底
                    val entity = LocalTemplateManager.loadLocalTempFile(
                            containerConfig?.module,
                            containerConfig?.templateId,
                            Constants.HTTPResCode.CODE_BACKUP_FROM_NEW_API,
                            mtaId
                    )
                    if (CommonUtil.isTemplateValid(entity)) {
                        DynamicSdk.handException(
                            IExceptionHandler.DynamicExceptionData.TYPE_LOCAL_BACKUP,
                            "DynamicContainer.createView 使用兜底",
                            containerConfig?.templateId,
                            containerConfig?.module,
                            Constants.LOAD_BACKUP_TYPE_IN_LOAD,
                            null
                        )
                        view = container.innerCreateView(
                                entity!!.viewNode,mtaId,
                                entity,source = "backUp",
                                time = loadTime
                        )
                    }
                }
            }
            return view
        }

        public override fun onPostExecute(view: View?) {
            if(view != null){
                var time = System.nanoTime()
                val baseFrameLayout = view.findViewById<BaseFrameLayout>(R.id.dynamic_root_view)
                baseFrameLayout?.addLifecycleObserver()
                LogUtil.e("createView","addLifecycleObserver time = "+DynamicMtaUtil.getCurMicroseconds(System.nanoTime() - time))
                time = System.nanoTime()
                if(container.isAsyncCreateView){
                    //异步创建的先不可见，等待数据绑定完成
                    view.visibility = INVISIBLE
                }
                if(view is YogaLayout && view.yogaLayoutLayoutParams !=null){
                    container.addView(view, LayoutParams(view.yogaLayoutLayoutParams))
                }else{
                    container.addView(view, LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT))
                }
                LogUtil.e("createView","addView time = "+DynamicMtaUtil.getCurMicroseconds(System.nanoTime() - time))
                listener?.onDynamicViewCreate()
            }else{
                listener?.onError(DynamicException("dynamic view is null"))
            }

        }


    }
}