package com.jd.dynamic.base

import android.app.Activity
import android.text.TextUtils
import androidx.annotation.NonNull
import com.jd.dynamic.apis.*
import com.jd.dynamic.lib.dynamic.cache.NewCacheManager
import com.jd.dynamic.lib.utils.FakeDyContainer
import java.util.HashMap

/**
 * @author: xuepanjie1
 * @date:2022/3/18
 * @description:
 */

class DynamicDriver : IDynamicDriver {
    private val lock = Any()
    private var cusFactory: HashMap<String, HashMap<String, IFunctionFactory>> = HashMap()

    override fun prepare(@NonNull module: String, @NonNull scene: String) {
        DynamicSdk.getEngine().newFetchTemplates(null, true, module)
    }

    fun createFakeContainer(config: DYContainerConfig) : FakeDyContainer {
        val container = FakeDyContainer()
        container.attachConfig(config)
        return container
    }

    override fun createContainer(@NonNull config: DYContainerConfig): DynamicContainer? {
        if (checkConfig(config)) {
            val container = DynamicContainer(config.context)
            container.attachConfig(config)
            return container
        }
        return null
    }

    override fun createContainer(
        @NonNull config: DYContainerConfig,
        @NonNull listener: IDYContainerListener
    ): DynamicContainer? {
        if (checkConfig(config)) {
            val container = DynamicContainer(config.context)
            container.attachConfig(config)
            container.attachListener(listener)
            listener.onCreate()
            return container
        }
        return null
    }

    override fun getTemplateStatus(
        @NonNull module: String,
        @NonNull templateId: String
    ): DYTemplateStatus {
        val cache = NewCacheManager.hasTempFile(
            module,
            templateId
        )
        val backup = NewCacheManager.hasLocalBackTempFile(module, templateId)
        return DYTemplateStatusImpl(cache, backup)
    }

    override fun registerCustomFunctionFactory(
        sysCode: String,
        biz: String,
        factory: IFunctionFactory
    ): Boolean {
        synchronized(lock) {
            var old = cusFactory[sysCode]
            if (null == old) {
                old = HashMap()
            }

            // 目前是按照已经拥有则错误去设计的
            if (null != old[biz]) {
                return false
            }

            old[biz] = factory
            return true
        }
    }

    override fun getFunctionFactory(sysCode: String, biz: String): IFunctionFactory? {
        synchronized(lock) {
            val old = cusFactory[sysCode]
            return if (null == old) {
                null
            } else {
                old[biz]
            }
        }
    }

    private fun checkConfig(@NonNull config: DYContainerConfig): Boolean {
        if (config.context !is Activity) {
            return false
        }
        if (TextUtils.isEmpty(config.module)) {
            return false
        }
        if (TextUtils.isEmpty(config.templateId)) {
            return false
        }
        return true
    }

}