package com.jd.dynamic.engine.base.invoker

import android.os.Build
import com.jd.dynamic.DYConstants
import com.jd.dynamic.base.DynamicSdk
import com.jd.dynamic.base.DynamicTemplateEngine
import com.jd.dynamic.base.interfaces.IUniConfigWithAdapter
import com.jd.dynamic.engine.JSCDynContext
import com.jd.dynamic.lib.common.Constants

/**
 * @author: xuepanjie1
 * @date:2023/2/24
 * @description:
 */

class AppContextInvoker(private val engine: DynamicTemplateEngine?) : Invoker {

    companion object {
        fun getAppContextInfo(inputType: Any?): Any {
            if (inputType !is String) {
                return DYConstants.DY_EMPTY_STR
            }

            return when (inputType) {
                "deviceType" -> Build.MODEL
                "osVersion" -> Build.VERSION.RELEASE
                "platform" -> DYConstants.DY_PLAT_ANDROID
                "appVersion" -> DynamicSdk.getEngine().context.packageManager
                    .getPackageInfo(DynamicSdk.getEngine().context.packageName, 0).versionName
                "currentMode" -> {
                    val config = DynamicSdk.getEngine().uniConfig
                    if (config is IUniConfigWithAdapter) {
                        return config.themeMode
                    } else {
                        return DYConstants.DY_EMPTY_STR
                    }
                }
                "fontSizeLevel" -> {
                    val config = DynamicSdk.getEngine().uniConfig
                    if (config is IUniConfigWithAdapter) {
                        return config.fontSizeLevel
                    } else {
                        return DYConstants.DY_EMPTY_STR
                    }
                }
                "darkMode" -> DynamicSdk.getEngine().dynamicDark.isDarkMode
                else -> DYConstants.DY_EMPTY_STR
            }
        }
    }

    override fun getName(): String {
        return Constants.FUNC_APP_CONTEXT
    }

    override fun onInvoke(
        jscDynContext: JSCDynContext?,
        methodName: String?,
        vararg params: Any?
    ): Any {
        DYConstants.DYLog("method name is $methodName params is $params")
        if (null == params || params.isEmpty()) {
            return DYConstants.DY_EMPTY_STR
        }
        return getAppContextInfo(params[0])
    }
}