package com.jd.dynamic.lib.expv2

import com.jd.dynamic.DYConstants
import com.jd.dynamic.lib.expv2.nodes.*
import com.jd.dynamic.lib.expv2.utils.XPStringScanner
import java.util.*
import java.util.concurrent.atomic.AtomicBoolean

/**
 * @author: xuepanjie1
 * @date:2022/1/7
 * @description: 共同的部分放到这里，具体使用方自定义的东西可以在实现
 */

abstract class XPBaseDriver : IXPDriver {
    companion object {
        private val left = charArrayOf(SYMBOL_BIG_BRACKET_L, SYMBOL_SMALL_BRACKET_L)
        private val flags = charArrayOf(
            SYMBOL_BIG_BRACKET_L,
            SYMBOL_SMALL_BRACKET_L,
            SYMBOL_BIG_BRACKET_R,
            SYMBOL_SMALL_BRACKET_R
        )
    }

    private val stack: Stack<XPNode> = Stack()

    private var expRoot: XPNode? = null

    abstract fun getNodeFactory(): IXPNodeFactory

    override fun parse(exp: String): Any? {
        val startTime = System.nanoTime()
        val nodeFactory = getNodeFactory()
        val kk = exp.trim()
        if (kk.startsWith(SYMBOL_EVENT_FUNC)) {
            return kk
        }

        val xpScanner = XPStringScanner(kk)
        while (xpScanner.canScan()) {
            val result = xpScanner.scanDataWithFlags(flags)?.trim()
            result?.let {
                if (it.contains(left[0]) || it.contains(left[1])) {
                    if (1 == it.length) {
                        pushNode(nodeFactory, it, null)
                    } else {
                        val start = 0
                        // 这里的 pos 是为了确定 flag 和 content 的位置
                        var pos: Int
                        pos = XPNodeFactory.takeValuePos(it)
                        if (pos < 0) {
                            pos = XPNodeFactory.conditionPos(it)
                        }
                        if (pos < 0) {
                            pos = XPNodeFactory.eventFuncPos(it)
                        }
                        when {
                            pos == 0 -> {
                                pushNode(nodeFactory, it, null)
                            }
                            pos > 0 -> {
                                pushNode(nodeFactory, it.substring(pos), it.substring(start, pos))
                            }
                            XPNodeFactory.isFunc(it) -> {
                                val loc = xpScanner.reverseSearch(it, SYMBOL_DOLLAR)
                                pushNode(nodeFactory, it.substring(loc), it.substring(start, loc))
                            }
                            else -> {
                                pushNode(
                                    nodeFactory,
                                    it.substring(it.length - 1),
                                    it.substring(0, it.length - 1)
                                )
                            }
                        }
                    }
                } else {
                    if (1 == it.length) {
                        popNode(nodeFactory, it, null)
                    } else {
                        popNode(
                            nodeFactory,
                            it.substring(it.length - 1),
                            it.substring(0, it.length - 1)
                        )
                    }
                }
            }
        }


        if (stack.size > 0) {
            return EMPTY_STR
        }

        DYConstants.DYLog("+++---+++ parse time is : ${System.nanoTime() - startTime}")

        return expRoot
    }


    private fun pushNode(
        nodeFactory: IXPNodeFactory,
        flagNoTrim: String,
        contentNoTrim: String?
    ) {
        val startTime = System.nanoTime()
        val flag = flagNoTrim.trim()
        val content = contentNoTrim?.trim()
        var contentNode: XPNode? = null
        content?.let {
            contentNode = nodeFactory.createNode(content)
        }
        val node = nodeFactory.createNode(flag)
        val endCreateTime = System.nanoTime()
        var stackNullTime = 0L
        var endAddTime = 0L
        var endPushTime = 0L
        if (stack.isEmpty()) {
            if (node is XPRootNode && node.canBeRoot()) {
                stack.push(node)
            }
            stackNullTime = System.nanoTime()
        } else {
            val root = stack.peek()
            if (root is XPRootNode) {
                contentNode?.let {
                    root.addChild(it)
                }
                root.addChild(node)
            }

            endAddTime = System.nanoTime()

            if (node.canBeRootNode()) {
                stack.push(node)
            }

            endPushTime = System.nanoTime()
        }

        DYConstants.DYLog("time create:${endCreateTime-startTime} " +
                "stackNull : ${stackNullTime - endCreateTime} + endAdd: ${endAddTime - endCreateTime}" +
                " + endPush :${endPushTime - endAddTime}")
    }

    private fun popNode(
        nodeFactory: IXPNodeFactory,
        flagNoTrim: String,
        contentNoTrim: String?
    ) {
        val flag = flagNoTrim.trim()
        val content = contentNoTrim?.trim()
        if (stack.isEmpty()) {
            DYConstants.DYLog("stack is null ")
            return
        }
        val root = stack.peek() as XPRootNode
        if (!root.isMatch(flag)) {
            DYConstants.DYLog("not match")
            return
        }
        if (!content.isNullOrEmpty()) {
            root.addChild(nodeFactory.createNode(content))
        }
        stack.pop()
        if (stack.isEmpty()) {
            expRoot = root
        }
    }
}