package com.jd.dynamic.lib.expv2.evaluators

import android.view.View
import com.jd.dynamic.base.DynamicTemplateEngine
import com.jd.dynamic.lib.expv2.EMPTY_STR
import com.jd.dynamic.lib.expv2.needRemovePH
import com.jd.dynamic.lib.expv2.nodes.XPNode
import com.jd.dynamic.lib.expv2.evaluators.base.XPEvaluatorWithAll
import com.jd.dynamic.lib.expv2.evaluators.util.ResultContainer
import com.jd.dynamic.lib.utils.FunctionDispatcher

/**
 * @author: xuepanjie1
 * @date:2021/12/31
 * @description: 解析 fun{} 类型的数据，目前只支持直接调用，params 那个不能用
 */

class EventFuncEvaluator(
    engine: DynamicTemplateEngine? = null,
    obj: Any? = null,
    view: View? = null
) : XPEvaluatorWithAll(engine, obj, view) {

    companion object {
        public const val SMALLEST_LENGTH = 5
    }

    override fun eval(node: XPNode): Any? {
        val content = node.parseContent()
        if (content is String) {
            val result = FunctionDispatcher.dispatcherFunction(
                getEventFuncId(content, node),
                obj, engine, view
            )
            return ResultContainer(result)
        }
        return content
    }

    private fun getEventFuncId(exp: String, node: XPNode): String {
        val length = exp.length
        if (length <= SMALLEST_LENGTH) {
            return EMPTY_STR
        }
        val event = exp.substring(4, length - 1)
        return if (event.needRemovePH()) {
            val origin = node.restore(event)
            if (origin is String) {
                origin
            } else {
                EMPTY_STR
            }
        } else {
            event
        }
    }
}