package com.jd.dynamic.lib.expv2.nodes

import android.text.TextUtils
import android.view.View
import com.jd.dynamic.DYConstants
import com.jd.dynamic.base.CachePool
import com.jd.dynamic.base.DynamicSdk
import com.jd.dynamic.base.DynamicTemplateEngine
import com.jd.dynamic.base.interfaces.IExceptionHandler
import com.jd.dynamic.base.interfaces.IUniConfigWithAdapter
import com.jd.dynamic.base.timer.TimerRequest
import com.jd.dynamic.engine.base.invoker.AppContextInvoker
import com.jd.dynamic.lib.common.Constants
import com.jd.dynamic.lib.dynamic.parser.DynamicXParser
import com.jd.dynamic.lib.expv2.*
import com.jd.dynamic.lib.expv2.evaluators.FunctionEvaluator
import com.jd.dynamic.lib.expv2.evaluators.TakeValueEvaluator
import com.jd.dynamic.lib.expv2.evaluators.util.TakeValueHelper
import com.jd.dynamic.lib.utils.CommonUtil
import com.jd.dynamic.lib.utils.FunctionDispatcher
import org.json.JSONArray
import org.json.JSONException
import org.json.JSONObject

/**
 * @author: xuepanjie1
 * @date:2021/12/28
 * @description: 方法节点，例如：$dark(a,b)
 */

class FunctionNode(flag: String) : XPRootNode(flag) {
    fun getFlag() : String {
        return head
    }

    override fun isMatch(tail: String): Boolean {
        this.tail = tail
        return tail == SYMBOL_SMALL_BRACKET_R.toString()
    }

    companion object {
        private const val K_DOT = ','
        private const val K_FUNC_DARK = "\$dark("
        private const val K_FUNC_DIC = "\$dic("
        private const val K_FUNC_JOINT = "\$joint("
        private const val K_FUNC_CALC = "\$calc("
        private const val K_FUNC_PLACE_HOLDER = "\$" + Constants.FUNC_PLACE_HOLDER + "("
        private const val K_FUNC_SIZE_ADAPTER = "\$sizeAdapter("
        private const val K_FUNC_MODE_ADAPTER = "\$modeAdapter("
        private const val K_FUNC_APP_CONTEXT = "\$" + Constants.FUNC_APP_CONTEXT + "("
        private const val K_FUNC_EVAL_FUN = "\$" + Constants.FUNC_EVAL_FUN + "("
        private const val K_FUNC_TIMER_FUN = "\$" + Constants.FUN_TYPE_TIMER + "("
        private const val K_PARAM_PLAT = "platform"
        private const val K_PARAM_SPACE = "space"
        const val K_PARAM_COMMA = "comma"
    }

    override fun evalResultInner(
        obj: Any?,
        engine: DynamicTemplateEngine?,
        targetView: View?
    ): Any? {
        val type = getFlag()
        val content = super.evalResultInner(obj, engine, targetView)
        if (null == content || content!is String||  content.length < type.length + 1) {
            return content
        }
        val param = content.substring(type.length, content.length - 1)
        return dealWithType(type, param, this, obj, engine, targetView)
    }

    private fun dealSingleParam(param: String, node: XPNode): Any {
        if (param.needRemovePH()) {
            val origin = node.restore(param)
            if (origin !is String || origin.isEmpty()) {
                return EMPTY_STR
            }
            return origin
        } else {
            return param
        }
    }

    private fun dealSingleType(type: String, param: String, node: XPNode, obj: Any?,
                               engine: DynamicTemplateEngine?, view: View?): Any? {
        val paramNew = dealSingleParam(param, node)
        return if (paramNew !is String || paramNew.isEmpty()) {
            EMPTY_STR
        } else when (type) {
            K_FUNC_CALC -> dealCalc(paramNew)
            K_FUNC_APP_CONTEXT -> dealDynamicInfo(paramNew)
            K_FUNC_PLACE_HOLDER -> dealPlaceHolder(paramNew)
            K_FUNC_EVAL_FUN -> dealEvalFun(paramNew, obj, engine, view)
            else -> EMPTY_STR
        }
    }

    private fun isSingleType(type: String): Boolean {
        return K_FUNC_CALC.equals(type, true)
                || K_FUNC_APP_CONTEXT.equals(type, true)
                || K_FUNC_PLACE_HOLDER == type
                || K_FUNC_EVAL_FUN == type
    }

    private fun dealWithType(
        type: String, param: String, node: XPNode, obj: Any?,
        engine: DynamicTemplateEngine?, view: View?
    ): Any? {
        if (isSingleType(type)) {
            return dealSingleType(type, param, node, obj, engine, view)
        }

        // 这一步就复原了值，有没有问题？
        val params = splitFuncParams(param, node)
        if (params.isEmpty()) {
            return EMPTY_STR
        }
        return when (type) {
            K_FUNC_DARK -> dealDark(params)
            K_FUNC_DIC -> dealDic(params)
            K_FUNC_JOINT -> dealJoint(params)
            K_FUNC_SIZE_ADAPTER -> dealSizeAdapter(params, engine)
            K_FUNC_MODE_ADAPTER -> dealModeAdapter(params, engine)
            K_FUNC_TIMER_FUN -> dealTimerData(params,engine)
            else -> {
                DYConstants.DYLog("this type not support $type")
                EMPTY_STR
            }
        }
    }
    private fun dealTimerData(params: List<Any?>,engine: DynamicTemplateEngine?): Any? {
        if(params.size == 2){
            val id = TimerRequest.getTimerId(params[0] as String,engine)
            val type = params[1] as String
            val allTimerData = engine?.cachePool?.getData(Constants.TIMER_DATA) as? String
            if(TextUtils.isEmpty(allTimerData)){
                return if(type == Constants.TIMER_VALUE){
                    ""
                }else{
                    0
                }
            }
            try {
                val timerJson = JSONObject(allTimerData!!)

                val data = timerJson.optJSONObject(id)
                return if(type == Constants.TIMER_VALUE){
                    data?.optString(Constants.TIMER_CURRENT_VALUE) ?:""
                }else{
                    data?.optInt(Constants.TIMER_STATE, 0) ?: 0
                }

            } catch (e: Exception) {
            }
        }
        return ""
    }
    private fun dealDark(params: List<Any?>): Any? {
        var result: Any? = null
        result = if (params.size < 2) {
            params[0]
        } else {
            val isDark: Boolean = DynamicSdk.getEngine().dynamicDark.isDarkMode
            if (!isDark) {
                params[0]
            } else {
                params[1]
            }
        }
        return if (DYConstants.DY_NULL_STR == result) EMPTY_STR else result
    }

    private fun dealDic(params: List<Any?>): Any {
        val json = JSONObject()
        params.forEach { p ->
            if (null != p && p is String && p.contains(SYMBOL_COLON)) {
                val first = p.indexOf(SYMBOL_COLON)
                var key: String? = null
                var value: String? = null
                if (first > 0 && first < p.length) {
                    key = p.substring(0, first).trim()
                    value = p.substring(first + 1).trim()
                }
                try {
                    if (!key.isNullOrEmpty()) {
                        if (value.notIllegal()) {
                            json.put(key, value)
                        } else {
                            json.put(key, EMPTY_STR)
                        }
                    }
                } catch (e: JSONException) {
                    e.printStackTrace()
                }
            }
        }
        return json
    }

    private fun dealJoint(params: List<Any?>): Any {
        val sb = StringBuilder()
        params.forEach { param ->
            param?.let {
                if (param.notIllegal()) {
                    sb.append(param)
                }
            }
        }
        return sb.toString()
    }

    private fun dealCalc(exp: String): Any? {
        val origin = MathCalc().calcExpr(exp)
        if (origin - origin.toInt() == 0.0) {
            return origin.toInt()
        }
        return origin
    }

    private fun dealModeAdapter(params: List<Any?>, engine: DynamicTemplateEngine?): Any? {
        val config = DynamicSdk.getEngine().uniConfig
        if (null == config || config !is IUniConfigWithAdapter) {
            DynamicSdk.handException(
                IExceptionHandler.DynamicExceptionData.TYPE_PARSE,
                "FunctionEvaluator parseAttribute modeAdapter error",
                CommonUtil.getExceptionBizField(engine),
                CommonUtil.getExceptionSystemCode(engine),
                Constants.HTTPResCode.CODE_PARSE_ADAPTER,
                IllegalStateException("IUniConfigWithFontAdapter must not null.")
            )
            return DYConstants.DY_EMPTY_STR
        }
        val modes = ArrayList<String>()
        params.forEach { p ->
            p?.let {
                modes.add(p.toString())
            }
        }
        return config.getModeInfo(modes)
    }

    private fun str2IntAndPush(str: String, fonts: MutableList<Float>, engine: DynamicTemplateEngine?) {
        try {
            val result = str.trim().toFloat()
            fonts.add(result)
        } catch (e: NumberFormatException) {
            fonts.add(DYConstants.DY_INT_NEGATIVE_ONE.toFloat())
            DynamicSdk.handException(
                IExceptionHandler.DynamicExceptionData.TYPE_PARSE,
                "FunctionEvaluator parseAttribute sizeAdapter error",
                CommonUtil.getExceptionBizField(engine),
                CommonUtil.getExceptionSystemCode(engine),
                Constants.HTTPResCode.CODE_PARSE_ADAPTER, e
            )
            e.printStackTrace()
        }
    }

    private fun dealSizeAdapter(params: List<Any?>, engine: DynamicTemplateEngine?): Any {
        val config = DynamicSdk.getEngine().uniConfig
        if (null == config || config !is IUniConfigWithAdapter) {
            DynamicSdk.handException(
                IExceptionHandler.DynamicExceptionData.TYPE_PARSE,
                "FunctionEvaluator parseAttribute sizeAdapter error",
                CommonUtil.getExceptionBizField(engine),
                CommonUtil.getExceptionSystemCode(engine),
                Constants.HTTPResCode.CODE_PARSE_ADAPTER,
                IllegalStateException("IUniConfigWithFontAdapter must not null.")
            )
            return DYConstants.DY_INT_NEGATIVE_ONE
        }
        val fonts = ArrayList<Float>()
        params.forEach { param ->
            if (param is String) {
                str2IntAndPush(param, fonts, engine)
            }
        }
        return config.getTextSize(fonts)
    }

    private fun dealDynamicInfo(param: String): Any {
        return AppContextInvoker.getAppContextInfo(param)
    }

    private fun dealPlaceHolder(param: String): Any {
        if (K_PARAM_SPACE.equals(param, true)) {
            return SPACE_STR
        }
        if (K_PARAM_COMMA == param) {
            return ","
        }
        return EMPTY_STR
    }

    private fun dealEvalFun(param: String, obj: Any?,
                            engine: DynamicTemplateEngine?, view: View?): Any? {
        return FunctionDispatcher.dispatcherFunction(
            param,
            obj, engine, view
        )
    }

    private fun splitFuncParams(param: String, node: XPNode): List<Any?> {
        if (!param.contains(K_DOT)) {
            val pp = if (param.needRemovePH()) node.restore(param)
            else param
            return arrayListOf(pp)
        }

        val pp = param.toCharArray()
        val positions = ArrayList<Int>()
        var bracket = 0
        pp.forEachIndexed { index, it ->
            if (SYMBOL_SMALL_BRACKET_L == it) {
                bracket++
            }
            if (SYMBOL_SMALL_BRACKET_R == it) {
                bracket--
            }
            if (bracket == 0 && K_DOT == it) {
                positions.add(index)
            }
        }
        if (positions.size == 0) {
            val pp1 = if (param.needRemovePH()) node.restore(param)
            else param
            return arrayListOf(pp1)
        } else {
            positions.add(param.length)
            val result = ArrayList<Any?>()
            var start = 0
            for (end in positions) {
                val data = param.substring(start, end)
                val dd = if (data.needRemovePH()) node.restore(data) else data
                result.add(dd)
                start = end + 1
            }
            return result
        }
    }

}