package com.jd.dynamic.lib.expv2.nodes

import android.view.View
import com.jd.dynamic.DYConstants
import com.jd.dynamic.base.CachePool
import com.jd.dynamic.base.DynamicTemplateEngine
import com.jd.dynamic.lib.ab.DYABConfigUtil
import com.jd.dynamic.lib.expv2.*
import com.jd.dynamic.lib.expv2.evaluators.ConditionEvaluator
import com.jd.dynamic.lib.expv2.evaluators.TakeValueEvaluator
import com.jd.dynamic.lib.expv2.evaluators.util.ResultContainer
import com.jd.dynamic.lib.expv2.evaluators.util.TakeValueHelper
import org.json.JSONArray
import org.json.JSONObject

/**
 * @author: xuepanjie1
 * @date:2021/12/27
 * @description: path 取值节点，需要占位解析，例如 info.test.obj
 */

class PathNode(data: String) : XPRootNode(data) {
    override fun isMatch(tail: String): Boolean {
        this.tail = tail
        if (SYMBOL_BIG_BRACKET_R.toString() == tail) {
            return true
        }
        return false
    }

    override fun evalResultInner(
        obj: Any?,
        engine: DynamicTemplateEngine?,
        targetView: View?
    ): Any? {
        val takeStart = System.nanoTime()
        val content = super.evalResultInner(obj, engine, targetView)
        if (null == content || content !is String || content.length <= 3) {
            return content
        }

//        DYConstants.DYLog("evalResultInner parse content is : ${System.nanoTime() - takeStart}")
        val exp = ConditionEvaluator.getExp(content)
        return takeValue(obj, exp, engine, targetView)
    }

    private fun takeValue(obj: Any?, exp: String,
                          engine: DynamicTemplateEngine?,view: View?): Any? {

        if (!exp.notIllegal()) {
            return null
        }
        val objs = ArrayList<Any>()
        exp.split(TakeValueEvaluator.K_POINT).forEach { dot ->
            objs.add(dot)
        }
        if (obj is JSONArray) {
            return TakeValueEvaluator.takeValueFromJSONArray(obj, exp, this)
        }
        // 默认走这种方式取值。
        val tt = takeValueFromObj(obj, objs, this, engine, view)

        return tt
    }

    private fun takeValueFromObj(obj: Any?, strs: List<Any>, node: XPNode,
                                 engine: DynamicTemplateEngine?,view: View?): Any? {
        val enter = System.nanoTime()
        var objInner: Any? = if (obj !is View) obj else null
        val iter = strs.iterator()
        while (iter.hasNext()) {
            var key = iter.next()
            if (key is String && key.needRemovePH()) {
                val res = node.restore(key)
                if (null == res) {
                    continue
                } else {
                    key = res
                }
            }
            if (objInner is String) {
                if (FLAG_B == objInner.containBOrM()) {
                    try {
                        objInner = JSONObject(objInner)
                    } catch (e: Exception) {
                    }
                }
            }
            if (key is ResultContainer) {
                objInner = key.value
                continue
            }
            if (key is JSONObject || key is JSONArray) {
                objInner = key
                continue
            }
            if (key is String) {
                if (FLAG_B == key.containBOrM()) {
                    try {
                        objInner = JSONObject(key)
                        continue
                    } catch (e: Exception) {
                    }
                }
                if (key.contains(SYMBOL_MIDDLE_BRACKET_L)) {
                    var result: Pair<String, String>? = null
                    result = TakeValueEvaluator.str2Str(key)
                    if (result.first.isEmpty() || result.second.isEmpty()) {
                        return objInner
                    }
                    if (objInner !is JSONObject) {
                        return objInner
                    }
                    objInner = objInner.opt(result.first)
                    if (objInner is JSONArray) {
                        objInner =
                            TakeValueEvaluator.takeValueFromArray(objInner, result.second, node)
                    }
                    if (null == objInner) {
                        return objInner
                    }
                    continue
                }
                if (TakeValueEvaluator.isLength(key)) {
                    if (objInner is String) {
                        return objInner.length
                    }
                    if (objInner is JSONObject) {
                        try {
                            val lengthTemp = objInner.opt(key)
                            if (null != lengthTemp) {
                                objInner = lengthTemp
                                continue
                            }
                        } catch (e: Exception) {
                            e.printStackTrace()
                        }
                    }

                }
                val count = isCount(key)
                if (count.isNotEmpty()) {
                    if (TakeValueEvaluator.K_COUNT == count) {
                        if (objInner is JSONObject) {
                            try {
                                if (null != objInner.opt(key)) {
                                    objInner = objInner.opt(key)
                                    continue
                                } else {
                                    return objInner.length()
                                }
                            } catch (e: Exception) {
                                e.printStackTrace()
                            }
                        }
                    }

                    if (TakeValueEvaluator.K_COUNT_DOLLAR == count) {
                        if (objInner is JSONObject) {
                            return objInner.length()
                        }
                    }

                    return takeCount(objInner)
                }
                if (TakeValueEvaluator.K_CACHE.equals(key, true)) {
                    if (iter.hasNext()) {
                        var nextKey = iter.next()
                        if (nextKey is String && nextKey.needRemovePH()) {
                            val value = node.restore(nextKey)
                            if (null == value) {
                                return null
                            } else {
                                nextKey = value
                            }
                        }
                        if (nextKey is String) {
                            objInner = if (DYABConfigUtil.getInstance().isCacheUseObj) {
                                engine?.cachePool?.getDataObj(nextKey)
                            } else {
                                engine?.cachePool?.getData(nextKey)
                            }
                            if (null == objInner) {
                                return objInner
                            }
                        }
                    }
                    continue
                }
                if (TakeValueEvaluator.K_THIS.equals(key, true)) {
                    objInner = view
                    continue
                }
                if (TakeValueEvaluator.K_CONTAINER == key) {
                    var first:Any? = null
                    var sec:Any? = null

                    if (iter.hasNext()) {
                        first = iter.next()
                        if (iter.hasNext()) {
                            sec = iter.next()
                        }
                    }
                    return TakeValueEvaluator.dealEST(first, sec, engine)
                }
                if (TakeValueEvaluator.K_OBJECT.equals(key, true)) {
                    objInner = obj
                    continue
                }
                objInner = if (objInner is JSONObject) {
                    objInner.opt(key)
                } else if (objInner is JSONArray) {
                    val result = JSONArray()
                    val jsonArray = objInner
                    for (i in 0 until jsonArray.length()) {
                        val arrayObj = jsonArray[i]
                        if (arrayObj is JSONObject) {
                            result.put(arrayObj.opt(key))
                        } else {
                            continue
                        }
                    }
                    result
                } else {
                    val attrMethod = CachePool.getMethod(key)
                    if (attrMethod != null) {
                        val result = TakeValueHelper.takeValueFromMethod(objInner,
                            attrMethod, engine)
                        DYConstants.DYLog("result is : $result")
                        result
                    } else {
                        null
                    }
                }
            }
            if (null == objInner) {
                return objInner
            }
        }
        DYConstants.DYLog("take value use time is : ${System.nanoTime() - enter}")
        return objInner
    }

    private fun isCount(key: String): String {
        if (TakeValueEvaluator.K_COUNT.equals(key, true)) {
            return TakeValueEvaluator.K_COUNT
        }
        if (TakeValueEvaluator.K_COUNT_DOLLAR.equals(key, true)) {
            return TakeValueEvaluator.K_COUNT_DOLLAR
        }
        return EMPTY_STR
    }


    private fun takeCount(objInner: Any?) : Any? {
        return when (objInner) {
            is JSONArray -> {
                objInner.length()
            }
            is JSONObject -> {
                objInner.length()
            }
            is String -> {
                objInner.length
            }
            else -> {
                objInner
            }
        }
    }
}