package com.jd.dynamic.lib.expv2.nodes


import com.jd.dynamic.lib.expv2.INVALID_POS
import com.jd.dynamic.lib.expv2.SYMBOL_BIG_BRACKET_L
import com.jd.dynamic.lib.expv2.SYMBOL_BIG_BRACKET_R
import com.jd.dynamic.lib.expv2.SYMBOL_TAG_INTERMEDIATE_VALUE

import android.view.View
import com.jd.dynamic.DYConstants
import com.jd.dynamic.base.DynamicTemplateEngine

import com.jd.dynamic.lib.expv2.*

import com.jd.dynamic.lib.expv2.evaluators.IXPEvaluatorFactory
import com.jd.dynamic.lib.expv2.utils.flagIndex

/**
 * @author: xuepanjie1
 * @date:2021/12/27
 * @description: 基础节点，拥有共有的各种操作。
 */

abstract class XPNode(data: String) : XPBaseNode() {
    companion object {
        fun createPlaceHolder(index: Int): String {
            return DOUBLE_SYMBOL_PH + index.toString() + DOUBLE_SYMBOL_PH_R
        }
    }

    private var factory: IXPEvaluatorFactory? = null

    protected var head: String = data
    protected var tail: String? = null

    protected var content: String? = null

    private var boundResult: Any? = null

    fun attachFactory(factory: IXPEvaluatorFactory) {
        this.factory = factory
    }

    fun getBoundResult(): Any? {
        return boundResult
    }

    private fun setBoundResult(result: Any?) {
        this.boundResult = result
    }

    var boundResults: MutableList<Any>? = null

    private var immResult: Any? = null

    open fun intermediateResult(): Any? {
        if (null != immResult) {
            return immResult
        }
        val parser = factory?.getParser(this)
        val result = parser?.eval(this)
        if (null == result || result is Number) {
            immResult = result
            return result
        }
        if (result is String && result.isEmpty()) {
            immResult = result
            return result
        }
        setBoundResult(result)
        immResult = SYMBOL_TAG_INTERMEDIATE_VALUE
        return immResult
    }

    abstract fun isMatch(tail: String): Boolean

    abstract fun parseContent(): String?

    fun hasPlaceHolder(): Boolean {
        return false
    }

    fun restore(origin: Any?): Any? {
        if (null == origin) {
            return DYConstants.DY_EMPTY_STR
        }
        if (origin !is String || boundResults.isNullOrEmpty()) {
            return origin
        }

        val left = flagIndex(origin, DOUBLE_SYMBOL_PH)
        if (INVALID_POS == left.pos) {
            return origin
        }
        val right = flagIndex(origin, DOUBLE_SYMBOL_PH_R)
        if (INVALID_POS == right.pos || left.pos >= right.pos) {
            return origin
        }

        if (left.pos == 0 && right.pos == origin.length - 1) {
            val index = origin.substring(1, right.pos).toInt()
            if (index <= INVALID_POS || index > boundResults!!.size) {
                return null
            }
            return boundResults!![index]
        }

        var result: String = origin
        boundResults?.let {
            for (i in 0 until it.size) {
                result = result.replace(createPlaceHolder(i), it[i].toString())
            }
        }

        return result
    }

    fun parseResult(): Any? {
        var result = intermediateResult()
        if (null != this.boundResult) {
            result = this.boundResult
        }
        if (result is Number) {
            return result.toString()
        }
        return restore(result)
    }

    open fun canBeRootNode(): Boolean {
        return true
    }

    open fun ifShouldAttach(): Boolean {
        return false
    }

    open fun fallthroughBoundResults(): Boolean {
        return false
    }


    open fun releaseImmResult() {
        immResult = null

        boundResult = null

        boundResults?.clear()

        content = null
    }

    fun getResult(
        obj: Any?,
        engine: DynamicTemplateEngine?,
        targetView: View?
    ): Any? {
        var result = evalValue(obj, engine, targetView)
        if (null != this.boundResult) {
            result = this.boundResult
        }
        if (result is Number) {
            return result.toString()
        }
        return restore(result)
    }


    // 俩方法，一个是外部调用，一个是子 node 出结果，需要区分，这个可以作为循环子 view 时候的调用
    open fun evalValue(obj: Any?,
                       engine: DynamicTemplateEngine?,
                       targetView: View?) : Any? {
        if (null != immResult) {
            return immResult
        }
        val result = evalResultInner(
            obj,
            engine, targetView

        )
        if (null == result || result is Number) {
            immResult = result
            return result
        }
        if (result is String && result.isEmpty()) {
            immResult = result
            return result
        }

        setBoundResult(result)
        immResult = SYMBOL_TAG_INTERMEDIATE_VALUE
        return immResult
    }


    // 这个是最后的具体计算，全部的节点去替换这个
    abstract fun evalResultInner(obj: Any?,
                                 engine: DynamicTemplateEngine?,
                                 targetView: View?) : Any?
}