package com.jd.dynamic.lib.expv2.nodes

import com.jd.dynamic.DYConstants
import com.jd.dynamic.lib.expv2.*
import com.jd.dynamic.lib.expv2.evaluators.IXPEvaluatorFactory

/**
 * @author: xuepanjie1
 * @date:2021/12/28
 * @description: 负责 node 制造，输入的 flag 都是 xxx{ 或者 xxx( content 都在前面
 */
class XPNodeFactory(private val factory: IXPEvaluatorFactory) : IXPNodeFactory {
    companion object {
        fun eventFuncPos(flag: String): Int {
            return flag.lastIndexOf(SYMBOL_EVENT_FUNC)
        }

        fun conditionPos(flag: String): Int {
            return flag.lastIndexOf(SYMBOL_CONDITION)
        }

        fun takeValuePos(flag: String): Int {
            val chars = flag.toCharArray()
            if (flag.length >= 2 && chars[flag.length - 1] == '{'
                && chars[flag.length - 2] == '$') {
                return flag.length - 2
            } else {
                return -1
            }
//            return flag.lastIndexOf(SYMBOL_PATH)
        }

        fun isBracket(flag: String): Boolean {
            return flag.contains(SYMBOL_BIG_BRACKET_L) || flag.contains(SYMBOL_SMALL_BRACKET_L)
        }

        fun isFunc(flag: String): Boolean {
            return flag.length > 2 && flag.contains(SYMBOL_DOLLAR) && flag.endsWith(
                SYMBOL_SMALL_BRACKET_L
            )
        }

        val node = TextNode("")

        fun obtainNode() : XPNode {
            return node
        }
    }

    override fun createNode(flag: String): XPNode {
        // 播放器图标第一次不展示，和修改有关系吗。。。。。。。我擦
//        DYConstants.DYLog("create node flag is $flag")
        if (false) {
            return obtainNode()
        }
        val node = when {
            isFunc(flag) -> {
                FunctionNode(flag)
            }
            takeValuePos(flag) == 0 -> {
                PathNode(flag)
            }
            conditionPos(flag) == 0 -> {
                ConditionNode(flag)
            }
            eventFuncPos(flag) == 0 -> {
                EventFuncNode(flag)
            }
            isBracket(flag) -> {
                BracketNode(flag)
            }
            else -> {
                TextNode(flag)
            }
        }
        node.attachFactory(factory)
        return node
    }
}
