package com.jd.dynamic.lib.expv2.nodes

import android.view.View
import com.jd.dynamic.base.DynamicTemplateEngine
import com.jd.dynamic.lib.expv2.SYMBOL_TAG_INTERMEDIATE_VALUE

/**
 * @author: xuepanjie1
 * @date:2021/12/28
 * @description:
 */

abstract class XPRootNode(content: String) : XPNode(content) {
    private val LOCK = Any()

    private var childs: MutableList<XPNode>? = null


    override fun releaseImmResult() {
        childs?.let {
            for (child in it) {
                child.releaseImmResult()
            }
        }

        super.releaseImmResult()
    }

    fun addChild(child: XPNode) {
        synchronized(LOCK) {
            if (null == childs) {
                childs = ArrayList()
            }
            childs?.add(child)
        }
    }

    open fun canBeRoot(): Boolean {
        return true
    }

    override fun evalResultInner(
        obj: Any?,
        engine: DynamicTemplateEngine?,
        targetView: View?
    ): Any? {
//        val parseContentEnter = System.nanoTime()

        if (!content.isNullOrEmpty()) {
            return content
        }

        val sb = StringBuilder()
        sb.append(head)
//        DYConstants.DYLog("root evalResultInner add head time" +
//                " : ${System.nanoTime() - parseContentEnter}")
        childs?.let {
            for (child in it) {

                var result: Any? = null
                result = child.evalValue(obj, engine, targetView)
//                DYConstants.DYLog("root is : $result time is : ${System.nanoTime() - parseContentEnter}")

                if (null != result && result !is String) {
                    sb.append(result)
                    continue
                }

                val curCount = if (boundResults.isNullOrEmpty()) {
                    boundResults = ArrayList()
                    0
                } else boundResults!!.size

                if (SYMBOL_TAG_INTERMEDIATE_VALUE == result) {
                    child.getBoundResult()?.let { chResult ->
                        sb.append(createPlaceHolder(curCount))
                        boundResults?.add(chResult)
                    }
                    continue
                }

                if (!child.fallthroughBoundResults()) {
                    sb.append(result)
                    continue
                }

                var secSb = result as String
                val bs = child.boundResults
                bs?.forEachIndexed { index, s ->
                    val origin = createPlaceHolder(index)
                    val target = createPlaceHolder(curCount + index)
                    secSb = secSb.replace(origin, target)
                }
                bs?.apply {
                    boundResults?.addAll(bs)
                }
                sb.append(secSb)
            }
        }
        sb.append(tail)
        content = sb.toString()
        return content
    }

    override fun parseContent(): String? {
//        val parseContentEnter = System.nanoTime()
        if (!content.isNullOrEmpty()) {
            return content
        }

        val sb = StringBuilder()
        sb.append(head)
//        DYConstants.DYLog("parse add head time : ${System.nanoTime() - parseContentEnter}")
        childs?.let {
            for (child in it) {
                var result: Any? = null

                result = child.intermediateResult()

                if (null != result && result !is String) {
                    sb.append(result)
                    continue
                }

                val curCount = if (boundResults.isNullOrEmpty()) {
                    boundResults = ArrayList()
                    0
                } else boundResults!!.size

                if (SYMBOL_TAG_INTERMEDIATE_VALUE == result) {
                    child.getBoundResult()?.let { chResult ->
                        sb.append(createPlaceHolder(curCount))
                        boundResults?.add(chResult)
                    }
                    continue
                }

                if (!child.fallthroughBoundResults()) {
                    sb.append(result)
                    continue
                }

                var secSb = result as String
                val bs = child.boundResults
                bs?.forEachIndexed { index, s ->
                    val origin = createPlaceHolder(index)
                    val target = createPlaceHolder(curCount + index)
                    secSb = secSb.replace(origin, target)
                }
                bs?.apply {
                    boundResults?.addAll(bs)
                }
                sb.append(secSb)
            }
        }
        sb.append(tail)
        content = sb.toString()
        return content
    }
}