package com.jd.dynamic.lib.localRes

import android.text.TextUtils
import com.jd.dynamic.base.DynamicMtaUtil
import com.jd.dynamic.base.DynamicSdk
import com.jd.dynamic.base.interfaces.IExceptionHandler
import com.jd.dynamic.entity.ResultEntity
import com.jd.dynamic.lib.downgrade.ModuleConfig
import com.jd.dynamic.lib.dynamic.parser.NewDynamicXParser
import com.jd.dynamic.lib.utils.LogUtil
import com.jd.dynamic.lib.utils.ZipUtils
import java.io.File

object LocalTemplateManager {
    private const val TAG:String = "LocalTemplateManager"
    var tempTypeMapping:MutableMap<String,String> = HashMap<String,String>()
    /**
     * 是否有兜底文件
     * @param systemCode
     * @param bizField
     * @return
     */
    @JvmStatic
    fun hasLocalTempFile(systemCode: String?, bizField: String?): Boolean {
        if(TextUtils.isEmpty(systemCode) || TextUtils.isEmpty(bizField)){
            return false
        }

        val dynamicStatus = DynamicSdk.getDynamicStatus()
        if (TextUtils.equals(dynamicStatus, ModuleConfig.STATUS_CLOSE)) { //不可用,直接返回
            DynamicMtaUtil.uploadDowngradeTemplateMta(
                systemCode, bizField, "",
                dynamicStatus, null, null
            )
            return false
        }
        val moduleConfig = ModuleConfig.getOrCreateModuleConfig(systemCode)
        if (TextUtils.equals(moduleConfig.status, ModuleConfig.STATUS_CLOSE) //业务模块不可用、
            && !moduleConfig.isInOpenList(bizField) && !moduleConfig.isInLocalList(bizField) //并且当前模板不在可用列表里
        ) {
            DynamicMtaUtil.uploadDowngradeTemplateMta(
                systemCode, bizField, "",
                dynamicStatus, null, null
            )
            return false
        }

        if (!TextUtils.isEmpty(DynamicSdk.getEngine()?.localResourcePath)) {
            if(tempTypeMapping.containsKey(bizField)){
                return true
            }
            val localTempName = DynamicSdk.getEngine().localResourcePath + File.separator + systemCode
            try {
                val fileLists = DynamicSdk.getEngine().context.assets.list(localTempName)
                if (!fileLists.isNullOrEmpty()) {
                    for (file in fileLists) {
                        if (file == "$bizField.zip" || file == bizField) {
                            //记录一下后续方便知道类型加载模板
                            tempTypeMapping[bizField!!] = file
                            return true
                        }
                    }
                }
            } catch (e: Exception) {
            }
        }
        return false
    }

    /**
     * 加载兜底模板
     * @param systemCode  模块名
     * @param bizField    模板名
     * @param from        调用来源
     * @param mtaId       埋点Id
     * @param withLoadMta 是否上报load埋点，默认上报
     */
    @JvmStatic
    fun loadLocalTempFile(systemCode: String?, bizField: String?,from: Int, mtaId: String?,withLoadMta:Boolean=true): ResultEntity? {
        if(TextUtils.isEmpty(systemCode) || TextUtils.isEmpty(bizField)){
            return null
        }
        if(!tempTypeMapping.containsKey(bizField)){//没有缓存，可能之前没查询有没有兜底，这里查一遍
            hasLocalTempFile(systemCode, bizField)
        }
        if(tempTypeMapping.containsKey(bizField)){
            val fileName = tempTypeMapping[bizField]
            return if(fileName!!.endsWith(".zip")){
                loadZipLocalFile(systemCode!!, bizField!!, from, mtaId,withLoadMta)
            }else{
                val localTempName = DynamicSdk.getEngine().localResourcePath +
                        File.separator + systemCode +
                        File.separator + bizField
                LogUtil.d(TAG,"loadBinaryLocalFile","bizField is $bizField,path is $localTempName")
                NewDynamicXParser.parseBinaryToResultEntity(localTempName,true,bizField,systemCode,mtaId,withLoadMta)
            }
        }

        return null
    }
    private fun loadViewNode(systemCode: String, bizField: String,localPath: String,mtaId: String?,withLoadMta:Boolean): ResultEntity{
        return NewDynamicXParser.parseBinaryToResultEntity(localPath, bizField, systemCode,mtaId,withLoadMta)
    }
    private fun loadZipLocalFile(systemCode: String, bizField: String,from: Int, mtaId: String?,withLoadMta:Boolean): ResultEntity?{

        try {
            val localTempName = DynamicSdk.getEngine().localResourcePath +
                    File.separator + systemCode +
                    File.separator + "$bizField.zip"
            LogUtil.d(TAG,"loadZipLocalFile","bizField is $bizField,path is $localTempName")
            val unZipPath = getLocalTemplateUnzipPath(systemCode, bizField)
            val cacheDir = File(unZipPath)

            //存在并且是个目录，说明已经解压了
            return if(cacheDir.exists() && cacheDir.isDirectory && !cacheDir.list().isNullOrEmpty()){
                loadViewNode(systemCode,bizField,unZipPath,mtaId,withLoadMta)
            }else{
                val inputStream = DynamicSdk.getEngine().context.assets.open(localTempName)
                ZipUtils.unZipFileStream(inputStream, unZipPath)
                loadViewNode(systemCode,bizField,unZipPath,mtaId,withLoadMta)
            }
        }catch (e:Exception){
            DynamicSdk.handException(IExceptionHandler.DynamicExceptionData.TYPE_LOCAL_BACKUP_ERROR,"加载兜底文件失败",bizField,systemCode,from,e)
        }
        return null
    }
    /**
     * 获取兜底文件解压目录
     */
    private fun getLocalTemplateUnzipPath(systemCode: String, bizField: String): String {
        val version = DynamicSdk.getEngine().context.packageManager.getPackageInfo(DynamicSdk.getEngine().context.packageName,0).versionName
        return getLocalTemplateDirPath() +
                File.separator + version +
                File.separator + systemCode +
                File.separator + bizField
    }
    @JvmStatic
    fun getLocalTemplateDirPath():String{
        val localPath = "localTemp"
        val rootDir = DynamicSdk.getEngine().context.applicationContext.filesDir
        return rootDir.absolutePath + File.separator + DynamicSdk.getEngine().cacheDir +
                File.separator + localPath
    }
}