package com.jd.dynamic.lib.report.impl

import com.jd.dynamic.lib.common.Constants
import com.jd.dynamic.lib.report.*
import com.jd.dynamic.lib.report.interfaces.IReportTransform
import org.json.JSONObject
import java.lang.Exception

/**
 * @author: xuepanjie1
 * @date:2022/2/28
 * @description:
 */

class DynamicBaseTrans(private val module: String) :
    IReportTransform<ReportParam, JSONObject?> {
    override fun transform(origin: ReportParam): JSONObject? {
        val json = JSONObject()
        val cusEventTime = origin.eventTime
        return try {
            // update this time to get create
            if (cusEventTime is Long && cusEventTime > 0) {
                json.put(REPORT_EVENT_TIME, cusEventTime)
            } else {
                json.put(REPORT_EVENT_TIME, System.currentTimeMillis())
            }
            json.put(REPORT_TYPE, REPORT_TYPE_DEFAULT)
            json.put(REPORT_DATA, origin.data)
//            json.put(REPORT_DYNAMIC_PAGE_ID, origin.pageId)
//            json.put(REPORT_DYNAMIC_EVENT_ID, origin.eventId)
            json.put(REPORT_EVENT_ID, REPORT_MODULE_DEFAULT) // 使用默认值作为pageId和eventId为了防止漏斗
            json.put(REPORT_PAGE_ID, REPORT_MODULE_DEFAULT)
            if (origin.extJson != null) {
                val ext = JSONObject(origin.extJson)
                insertCreateTime2Ext(cusEventTime, ext)
                ext.put(Constants.EVENT_UUID, origin.uuid)
                ext.put(REPORT_DYNAMIC_PAGE_ID, origin.pageId)
                ext.put(REPORT_DYNAMIC_EVENT_ID, origin.eventId)
                json.put(REPORT_EXT, ext)
            } else {
                val ext = JSONObject()
                insertCreateTime2Ext(cusEventTime, ext)
                ext.put(Constants.EVENT_UUID, origin.uuid)
                ext.put(REPORT_DYNAMIC_PAGE_ID, origin.pageId)
                ext.put(REPORT_DYNAMIC_EVENT_ID, origin.eventId)
                json.put(REPORT_EXT, ext)
            }
            json.put(REPORT_MODULE, module)
        } catch (e: Exception) {
            null
        }
    }

    private fun insertCreateTime2Ext(cusTime: Long?, ext: JSONObject) {
        if (cusTime is Long && cusTime > 0) {
            ext.put(Constants.EVENT_CREATE_TIME, System.currentTimeMillis())
        }
    }
}