package com.jd.dynamic.lib.report.impl

import android.content.Context
import com.jd.dynamic.DYConstants
import com.jd.dynamic.base.DynamicSdk
import com.jd.dynamic.base.interfaces.INetWorkRequest
import com.jd.dynamic.lib.common.Constants
import com.jd.dynamic.lib.report.*
import com.jd.dynamic.lib.report.interfaces.IReportCleaner
import com.jd.dynamic.lib.report.interfaces.IReporter
import org.json.JSONArray
import org.json.JSONObject
import java.lang.Exception
import java.util.*

/**
 * @author: xuepanjie1
 * @date:2022/2/28
 * @description: 上报类，这里直接上报了，后期使用控制类去做，需要先持久化再上报.
 */

class Reporter(module: String) : IReporter, IReportCleaner {
    private val trans = DynamicBaseTrans(module)
    private val lock = Any()
    private val cacheEvent = LinkedList<JSONObject>()
    private var config = ReportConfig(Constants.DEFAULT_REPORT_COUNT, Constants.REPORT_MIN_GAP)

    companion object {
        private const val FUNCTION_ID_COLOR = "platEventCollect"

        fun reportEvent(json: JSONObject?, appId: String) {
            testForReport(json, appId)?.let {

                DynamicSdk.getEngine().request.requestWithHost(
                    FUNCTION_ID_COLOR, it.toString(),
                    DynamicSdk.getEngine().host,
                    object : INetWorkRequest.ResponseCallBack {
                        override fun onStart() {

                        }

                        override fun onSuccess(response: JSONObject?) {
//                            LogUtil.e("report2222","report22222 success json is $response .")
//                            DYConstants.DYLog("report22222 success res is $response")
                        }

                        override fun onError(error: INetWorkRequest.ErrorResponse?) {
//                            DYConstants.DYLog("report22222 on error . $error")
                        }
                    })
            }
        }

        private fun testForReport(jsonData: JSONObject?, appId: String): JSONObject? {
            if (jsonData == null) {
                return null
            }
            return try {
                val json = JSONObject()
                val events = JSONArray()
                events.put(jsonData)
                json.put(REPORT_EVENTS, events)
                json.put(REPORT_APP_ID, appId)
                json.put(REPORT_SOURCE, REPORT_SOURCE_DEFAULT)
                json
            } catch (e: Exception) {
                null
            }
        }
    }

    override fun attachConfig(config: ReportConfig) {
        this.config = config
    }

    fun getConfig() : ReportConfig {
        return config
    }

    override fun report(
        ctx: Context,
        appId: String,
        eventId: String,
        pageId: String,
        jsonData: String,
        extJson: String?
    ) {
        synchronized(lock) {
            val json = trans.transform(ReportParam(eventId, pageId, jsonData, extJson)) ?: return
            cacheEvent.add(json)
            if (cacheEvent.size > config.triggerCount) {
                reportImmediate(appId)
            }
        }
    }

    override fun reportWithEventTime(
        ctx: Context,
        appId: String,
        eventId: String,
        pageId: String,
        jsonData: String,
        time: Long,
        uuid: String,
        extJson: String?
    ) {
        synchronized(lock) {
            val json = trans.transform(ReportParam(eventId, pageId, jsonData, extJson, time, uuid)) ?: return
            cacheEvent.add(json)
            if (cacheEvent.size > config.triggerCount) {
                reportImmediate(appId)
            }
        }
    }

    override fun reportImmediate(appId: String) {
        synchronized(lock) {
            val reportEvents = getEvents() ?: return

            val target = createEvents(reportEvents, appId) ?: return

            DynamicSdk.getEngine().request.requestWithHost(
                FUNCTION_ID_COLOR, target.toString(),
                DynamicSdk.getEngine().host,
                ReporterCallbackImpl(reportEvents, this)
            )
        }
    }

    // 按照上报格式创建事件
    private fun createEvents(eventList: List<JSONObject>, appId: String): JSONObject? {
        return try {
            val json = JSONObject()
            val events = JSONArray()
            for (event in eventList) {
                events.put(event)
            }
            json.put(REPORT_EVENTS, events)
            json.put(REPORT_APP_ID, appId)
            json.put(REPORT_SOURCE, REPORT_SOURCE_DEFAULT)
            json
        } catch (e: Exception) {
            null
        }
    }

    // 获取缓存事件
    private fun getEvents(): List<JSONObject>? {
        synchronized(lock) {
            if (cacheEvent.isEmpty()) {
                return null
            }

            val events = LinkedList(cacheEvent)
            cacheEvent.clear()
            return events
        }
    }

    private class ReporterCallbackImpl(
        private val events: List<JSONObject>,
        private val cleaner: IReportCleaner
    ) : INetWorkRequest.ResponseCallBack {
        override fun onStart() {

        }

        override fun onSuccess(response: JSONObject?) {
//            cleaner.clean(events)
        }

        override fun onError(error: INetWorkRequest.ErrorResponse?) {

        }

    }

    override fun clean(events: List<JSONObject>) {
        synchronized(lock) {
            for (event in events) {
                cacheEvent.remove(event)
            }
        }
    }
}