package com.jd.dynamic.lib.storage.cleaner

import com.jd.dynamic.DYConstants
import com.jd.dynamic.lib.storage.IDYStorageCleaner
import com.jd.dynamic.lib.storage.IDYStorageFilterFile
import java.io.File

/**
 * @author: xuepanjie1
 * @date:2022/4/11
 * @description: 传入路径和对应的 filter ，如果filter了那么就删除对应的文件
 * 仅删除对应的路径下的非法文件，不处理子文件夹非法文件，如果有需要可以创建递归的 cleaner
 */

class DYIllegalFileCleaner(private val fileFilters: List<IDYStorageFilterFile>) :
    IDYStorageCleaner {
    override fun clean(path: String) {
        val cleanFile = File(path)
        if (!cleanFile.exists()) {
            return
        }
        if (!cleanFile.isDirectory) {
            return
        }
        val childFiles = cleanFile.listFiles() ?: return
        for (file in childFiles.iterator()) {
            if (file.isDirectory) {
                continue
            }

            var result = true
            for (filter in fileFilters) {
                if (!filter.filter(file)) {
                    result = false
                    break
                }
            }
            if (result) {
                DYIllegalFileRecursiveCleaner.reportDelFile(file)
                DYConstants.DYLog(" delete file -> file path: ${file.absolutePath} file name is : ${file.name} ," +
                        " length ${file.length()}")
                file.delete()
            }
        }
    }
}