package com.jd.dynamic.lib.storage.cleaner

import com.jd.dynamic.DYConstants
import com.jd.dynamic.base.DynamicSdk
import com.jd.dynamic.base.interfaces.IExceptionHandler
import com.jd.dynamic.lib.common.Constants
import com.jd.dynamic.lib.storage.IDYStorageCleaner
import com.jd.dynamic.lib.storage.IDYStorageFilterFile
import java.io.File

/**
 * @author: xuepanjie1
 * @date:2022/4/11
 * @description: filters 是且的条件，必须都满足才会 clean
 */

class DYIllegalFileRecursiveCleaner(private val fileFilters: List<IDYStorageFilterFile>) :
    IDYStorageCleaner {
    companion object {
        fun reportDelFile(file: File?) {
            file?.let {
                DynamicSdk.handException(
                    IExceptionHandler.DynamicExceptionData.TYPE_FILE,
                    "file name : ${file.name} length: ${file.length()} modify time: ${file.lastModified()}",
                    DYConstants.DY_EMPTY_STR,
                    DYConstants.DY_EMPTY_STR,
                    Constants.HTTPResCode.CODE_DEL_ILLEGAL_FILE,
                    null
                )
            }
        }
    }

    override fun clean(path: String) {
        DYConstants.DYLog("path is : $path")
        val cleanFile = File(path)
        if (!cleanFile.exists()) {
            return
        }
        if (!cleanFile.isDirectory) {
            return
        }
        innerClean(cleanFile)
    }

    private fun innerClean(cleanFile: File) {
        val childFiles = cleanFile.listFiles() ?: return
        for (file in childFiles.iterator()) {
            if (!file.exists()) {
                continue
            }

            if (file.isDirectory) {
                innerClean(file)
                continue
            }

            var result = true
            for (filter in fileFilters) {
                if (!filter.filter(file)) {
                    result = false
                    continue
                }
            }
            if (result) {
                reportDelFile(file)
                DYConstants.DYLog(" delete file -> file path: ${file.absolutePath} file name is : ${file.name} ," +
                        " length ${file.length()}")
                file.delete()
            }
        }
    }
}