package com.jd.dynamic.lib.utils

import android.app.Activity
import android.content.Context
import android.text.TextUtils
import android.util.AttributeSet
import android.widget.FrameLayout
import com.jd.dynamic.DYConstants
import com.jd.dynamic.apis.DYContainerConfig
import com.jd.dynamic.base.DynamicMtaUtil
import com.jd.dynamic.base.DynamicSdk
import com.jd.dynamic.base.DynamicTemplateEngine
import com.jd.dynamic.entity.MtaTimePair
import com.jd.dynamic.entity.ViewNode
import com.jd.dynamic.lib.common.Constants
import com.jd.dynamic.lib.dynamic.cache.NewCacheManager
import com.jd.dynamic.lib.dynamic.parser.NewDynamicXParser
import com.jd.dynamic.lib.localRes.LocalTemplateManager
import com.jd.dynamic.lib.viewparse.BaseFrameLayout
import java.io.File

/**
 * @author: xuepanjie1
 * @date:2022/5/13
 * @description:
 */

class FakeDyContainer {
    private val DY_DATA = "DynamicData"
    private val DY_EVENTS = "Events"
    private var container: FrameLayout? = null
    private var engine: DynamicTemplateEngine? = null
    private var containerConfig: DYContainerConfig? = null

    fun attachConfig(config: DYContainerConfig) {
        containerConfig = config
    }

    private fun isConfigInvalid(): Boolean {
        return containerConfig == null
    }

    /**
     * 0. 内存
     * 1. 磁盘缓存
     * 2. 兜底 & 请求网络
     * */
    // 仅仅从内存判断是否有 template 没有则加载失败。
    fun load(): Boolean {
        if (isConfigInvalid()) {
            return false
        }
        val loadTime = System.currentTimeMillis()
        DYConstants.DYLog("fake enter load.... load time $loadTime" +
                " module: ${containerConfig?.module} template: ${containerConfig?.templateId}")
        if (NewCacheManager.hasTempFile(
                containerConfig?.module,
                containerConfig?.templateId
            )
        ) {
            // 磁盘
            val template = NewCacheManager.getTemplateByBizField(
                containerConfig?.module,
                containerConfig?.templateId
            )
            template?.let {
                val mtaId = DynamicMtaUtil.startMtaStat(
                    DynamicSdk.getEngine().appType,
                    containerConfig?.module,
                    DYConstants.DY_EMPTY_STR,
                    containerConfig?.templateId,
                    false
                )

                val digestFileName = DigestUtils.getDigest(
                    it.downloadUrl,
                    it.downloadFileName
                )
                val cacheFile = File(
                    NewCacheManager.getSysCodeAbsPath(containerConfig?.module) +
                            File.separator + it.businessCode,
                    digestFileName
                )

                if (cacheFile.exists() && cacheFile.isFile) {
                    val viewNode = NewDynamicXParser.parseBinaryToViewNode(
                        cacheFile.absolutePath,
                        false,
                        containerConfig?.templateId,
                        containerConfig?.module,
                        mtaId,false
                    )
                    if (nodeValid(viewNode)) {
                        showViewNode(viewNode, mtaId, loadTime)
                        return true
                    }
                } else {
                    return false
                }
            }
        } else {
            // 兜底
            if (NewCacheManager.hasLocalBackTempFile(
                    containerConfig?.module,
                    containerConfig?.templateId
                )
            ) {

                val mtaId = DynamicMtaUtil.startMtaStat(
                    DynamicSdk.getEngine().appType,
                    containerConfig?.module,
                    DYConstants.DY_EMPTY_STR,
                    containerConfig?.templateId,
                    false
                )
                val entity = LocalTemplateManager.loadLocalTempFile( containerConfig?.module, containerConfig?.templateId,
                    Constants.HTTPResCode.CODE_BACKUP_FROM_NEW_API,"",false)
                if (CommonUtil.isTemplateValid(entity)){
                    showViewNode(entity!!.viewNode, mtaId, loadTime)
                    return true
                }
            }
            return false
        }
        return false
    }

    private fun nodeValid(viewNode: ViewNode?): Boolean {
        return null != viewNode && !TextUtils.isEmpty(viewNode.viewName)
    }

    private fun showViewNode(viewNode: ViewNode, mtaId: String, time: Long) {
        val start = System.currentTimeMillis()
        DYConstants.DYLog("fake showViewNode start time :$start time is $time hhh: ${start - time}")

        if (null == engine) {
            // 不需要每次创建
            engine = DynamicTemplateEngine(
                DYConstants.DY_EMPTY_STR,
                containerConfig?.context as? Activity,
                container,
                null
            )
        }

        engine?.shouldAutoListenDarkStatus(true)

        try {
            doInitFast(containerConfig?.context, viewNode, mtaId)
        } catch (e: Exception) {
            e.printStackTrace()
            DYConstants.DYLog("fast error!!!!")
        }

        engine?.release()
        DYConstants.DYLog("fake showViewNode end time : ${System.currentTimeMillis() - time}")
    }

    private fun doInitFast(context: Context?, rootNode: ViewNode?, mtaId: String) {
        if (rootNode == null) {
            DYConstants.DYLog("fast  error rootNode is null !!!!")
            return
        }
        DYConstants.DYLog("doInitFast new enter v2 !!!!")
        var viewNode: ViewNode? = null
        var eventNode: ViewNode? = null
        if (DY_DATA == rootNode.viewName) {
            viewNode = rootNode.getChildByName(DYConstants.DY_FLEXBOX_LAYOUT)
            eventNode = rootNode.getChildByName(DY_EVENTS)
        }
        if (viewNode == null) {
            viewNode = rootNode
        }
        val finalViewNode: ViewNode = viewNode
        val finalEventNode = eventNode
        val renderMtaTimePair = MtaTimePair.newInstance()
        renderMtaTimePair.startRecord()
        parseBFLayoutFast(
            context, finalViewNode,
            engine, finalEventNode
        )
        renderMtaTimePair.endRecord()
        DynamicMtaUtil.appendRenderMtaStat(mtaId, renderMtaTimePair)
    }

    private fun parseBFLayoutFast(
        context: Context?, finalViewNode: ViewNode,
        engine: DynamicTemplateEngine?,
        finalEventNode: ViewNode?
    ): BaseFrameLayout? {
        val baseFrameLayout = BaseFrameLayout(context, finalViewNode, engine,false)
        if (finalEventNode != null) {
            FunctionDispatcher.parseEvent(engine, finalEventNode)
        }
        return baseFrameLayout
    }
}